        // This test program works OK from VC++ 6.0
//-------------------------------------------------------------------------
// The "y1d4x_aa.dll" is a Win32 DLL containing procedures for optimizing
// cutting of any linear material.  It contains two similar procedures for
// solving so called "One-dimensional Bin Packing problem" - optimizing the
// distribution of any number of pieces (parts) to cut out having mostly
// different lengths, amongst a number of uniform or different stock items
// (stock lengths) in order to minimize material waste (or as a version -
// associated material cost).  Procedures Y1DPackB() and Y1DPackG() differ
// just by a structure of DLL calling arguments - input/output arrays.
//
// This is the main header file for the application.
// It declares the IDSC_DLL class in case of Demo y1d4x_aa.dll version(s).
//-------------------------------------------------------------------------

//! FOR CUSTOMIZED DLL, remember: (1) to correct UseDLLName 'y1d4n???.dll',
//! (2) to correct array limits below: BoxMax, BoxGrMax, CntMax, CntGrMax,
//! (3) to define proper RegCustomer/RegPassword in void InitVariables()

char*   UseDLLName="y1d4x_aa.dll";      // Define actual DLL file name !

//! BoxMax, CntMax..- array limits MUST be equal to BoxMax, CntMax.. in DLL
//! Activate below Const declarations in case of DEMO DLL version(s)!
const int BoxMax = 100;         // Demo DLL version(s)
const int CntMax = 100;
const int BoxGrMax = 20;
const int CntGrMax = 10;

//! Activate below Const declarations in case of WORKING DLL version(s)!
//const BoxMax = 5000;          // Working DLL version(s)
//const CntMax = 5000;
//const BoxGrMax = 1000;
//const CntGrMax = 100;

const int StockCheck = 555;     // Any values just for DLL transfer checks
const int PartCheck = 777;

int CallMode = 0; // 0 - Y1DPackG, group arrays, 1 - Y1DPackB, line arrays
long StockLthGr1; // 1-st stock group Length to modify in this demo

//--------------------------------------------------------------------
class IDSC_Dll
{
public:
         // ***  Current C++ Builder program working variables  ***

long StockNmb;          // Actual Stock / Part item numbers
long PartNmb;
long StockGrNmb;        // Actual Stock / Part group numbers
long PartGrNmb;

long ErrorNumb;         // DLL return error code, 0 = OK
long ErrorMsgs;         // DLL switch to display or not error messages - On/Off

char* RegCustomer;      // DLL input login data
long RegPassword;

long CostOptim;         // DLL input Optim.Settings (similar to 1DSC)
long IgnoreLast;
long EffortLev;
long ReduceLays;
long PrefShort;
long DiffStocks;
long DiffPieces;
long ReusLength;
long GapItIt;
long GapItEnd;
long GapGripEnd;
long OptimTime;
long IterMult;
long LayByMark;
long RunMsgMode;
long ReservInp2;

long CritSum;            // DLL output solution totals
long EffSum;
long StockSum;
long PartSum;
long LengthSum;
long WasteSum;
long CostSum;
long LastStockNo;
long LastParts;
long LastLength;
long LastWaste;
long LastCost;
long PatternSum;
long LayoutSum;
long LayoutCuts;
long ReservOut1;
long ReservOut2;

long  StockGrLth[CntGrMax + 1]; // DLL input arrays for Y1DPackG procedure
long  StockGrCst[CntGrMax + 1];
short StockGrQty[CntGrMax + 1];
long  PartGrLth[BoxGrMax + 1];
long  PartGrMrkID[BoxGrMax + 1];
short PartGrQty[BoxGrMax + 1];

long StockLength[CntMax + 1];   // DLL input arrays for Y1DPackB procedure
long StockCost[CntMax + 1];
long PartLength[BoxMax + 1];
long PartMarkID[BoxMax + 1];
                                // DLL output solution arrays (reference numbers)
short StockGrNo[CntMax + 1];
        //StockGrNo[j] = initial group reference No for Stock[j]
short PartGrNo[BoxMax + 1];
        //PartGrNo[i] = initial group reference No for Part[i]

                                // DLL output solution arrays (future results)
short PartStockNo[BoxMax + 1];
        // PartStockNo[i] = Stock No assigned to Part[i]
short StockLayNo[CntMax + 1];
        // StockLayNo[j] = Layout No assigned to Stock[j]
short StockPatNo[CntMax + 1];
        // StockPatNo[j] = Pattern No assigned to Stock[j]
short StockParts[CntMax + 1];
        // StockParts[j] = Total part number in Stock[j]
long StockWaste[CntMax + 1];
        //StockWaste[j] = Individual waste in Stock[j]

TCHAR Errmsg[100];      // Error message
//---------------------------------------------------------

void Init_function_Y1DPackB()
        	// Procedure to connect DLL library file and run Y1DPackB()
        {				 //Declare DLL procedure Y1DPackB
        typedef int (WINAPI *PFN_MyFunction)
        (long*, long*, char*, long,                     // Input
        long, long, long, long, long, long, long, long,
        long, long, long, long, long, long, long, long,
        long*, long*, long*, long*,                     // 4 Input arrays

        long*, long*, long*, long*, long*, long*, long*, long*,  // Output
        long*, long*, long*, long*, long*, long*, long*, long*, long*,
        short*, short*, short*, short*, long*);         // 5 Output arrays

        HINSTANCE hMyDll;

        if ((hMyDll=::LoadLibrary(UseDLLName))==NULL)
        {       /* Point library to the workspase, connect and
                   write its address in the variable hMyDll */
                wsprintf (Errmsg,TEXT("Check for DLL system error"));
        WinErr ();
        }
        else
        {       /* Declare variable pfnMyFunction1 and write address of
                   the procedure Y1DPackB in the variable pfnMyFunction */
        PFN_MyFunction pfnMyFunction;
        pfnMyFunction=(PFN_MyFunction)::GetProcAddress(hMyDll,"Y1DPackB");
                /* Run procedure Y1DPackB, all arguments belong
                                                to the class IDSC */
        (*pfnMyFunction)
        (&ErrorNumb, &ErrorMsgs, RegCustomer, RegPassword,
        CostOptim,   IgnoreLast, EffortLev,   ReduceLays,  // Input settings
        PrefShort,   DiffStocks, DiffPieces,  ReusLength,
        GapItIt,     GapItEnd,   GapGripEnd,  OptimTime,
        IterMult,    LayByMark,  RunMsgMode,  ReservInp2,
        StockLength, StockCost,  PartLength,  PartMarkID,  // 4 Input arrays

        &CritSum,    &EffSum,     &StockSum,   &PartSum,   // Output totals
        &LengthSum,  &WasteSum,   &CostSum,    &LastStockNo,
        &LastParts,  &LastLength, &LastWaste,  &LastCost,
        &PatternSum, &LayoutSum,  &LayoutCuts,
        &ReservOut1, &ReservOut2,                          // 5 Output arrays
        PartStockNo, StockLayNo,  StockPatNo, StockParts, StockWaste);

        FreeLibrary(hMyDll);   /* Disconnect DLL library */

        if (ErrorNumb != 0)        // To check for DLL solver internal errors
                  {wsprintf (Errmsg,TEXT("DLL reports Error: %i"),ErrorNumb);
                        WinErr ();
                  }
        else                       // To check for DLL data transfer errors
        {
    if (PartStockNo[PartNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /PartStockNo"));
                        WinErr ();
                  }
    if (StockLayNo[StockNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /StockLayNo"));
                        WinErr ();
                  }
    if (StockPatNo[StockNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /StockPatNo"));
                        WinErr ();
                  }
    if (StockParts[StockNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /StockParts"));
                        WinErr ();
                  }
    if (StockWaste[StockNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /StockWaste"));
                        WinErr ();
                  }
        }
        }
        };
//End of void Init_function_Y1DPackB()
//---------------------------------------------------------

void Init_function_Y1DPackG()
        	// Procedure to connect DLL library file and run Y1DPackG()
        {				 //Declare DLL procedure Y1DPackG
        typedef int (WINAPI *PFN_MyFunction)
        (long*, long*, char*, long,                     // Input
        long, long, long, long, long, long, long, long,
        long, long, long, long, long, long, long, long,
        long*, long*, short*, long*, long*, short*,     // 6 Input arrays

        long*, long*, long*, long*, long*, long*, long*, long*,  // Output
        long*, long*, long*, long*, long*, long*, long*, long*, long*,
        short*, short*, short*, short*, short*, short*, long*); // 7 Output arrays

        HINSTANCE hMyDll;

        if ((hMyDll=::LoadLibrary(UseDLLName))==NULL)
        {       /* Point library to the workspase, connect and
                   write its address in the variable hMyDll */
                wsprintf (Errmsg,TEXT("Check for DLL system error"));
        WinErr ();
        }
        else
        {       /* Declare variable pfnMyFunction1 and write address of
                   the procedure Y1DPackG in the variable pfnMyFunction1 */
        PFN_MyFunction pfnMyFunction1;
        pfnMyFunction1=(PFN_MyFunction)::GetProcAddress(hMyDll,"Y1DPackG");
                /* Run procedure Y1DPackG, all arguments belong
                                                to the class IDSC */
        (*pfnMyFunction1)
        (&ErrorNumb, &ErrorMsgs, RegCustomer, RegPassword,
        CostOptim,   IgnoreLast, EffortLev,   ReduceLays,  // Input settings
        PrefShort,   DiffStocks, DiffPieces,  ReusLength,
        GapItIt,     GapItEnd,   GapGripEnd,  OptimTime,
        IterMult,    LayByMark,  RunMsgMode,  ReservInp2,
        StockGrLth,  StockGrCst,   StockGrQty,             // 6 Input arrays
        PartGrLth,   PartGrMrkID,  PartGrQty,

        &CritSum,    &EffSum,     &StockSum,   &PartSum,   // Output totals
        &LengthSum,  &WasteSum,   &CostSum,    &LastStockNo,
        &LastParts,  &LastLength, &LastWaste,  &LastCost,
        &PatternSum, &LayoutSum,  &LayoutCuts,
        &ReservOut1, &ReservOut2,
        StockGrNo,    PartGrNo,                            // 7 Output arrays
        PartStockNo, StockLayNo,  StockPatNo, StockParts, StockWaste);

        FreeLibrary(hMyDll);   /* Disconnect DLL library */

        if (ErrorNumb != 0)        // To check for DLL solver internal error
                  {wsprintf (Errmsg,TEXT("DLL reports Error: %i"),ErrorNumb);
                        WinErr ();
                  }
        else                       // To check for DLL data transfer errors
        {
    if (PartStockNo[PartNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /PartStockNo"));
                        WinErr ();
                  }
    if (StockLayNo[StockNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /StockLayNo"));
                        WinErr ();
                  }
    if (StockPatNo[StockNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /StockPatNo"));
                        WinErr ();
                  }
    if (StockParts[StockNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /StockParts"));
                        WinErr ();
                  }
    if (StockWaste[StockNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /StockWaste"));
                        WinErr ();
                  }                                  //Checks for CallMode=0 case only
    if (PartGrNo[PartNmb + 1] != StockCheck + PartCheck)  
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /PartGrNo"));
                        WinErr ();
                  }
    if (StockGrNo[StockNmb + 1] != StockCheck + PartCheck)
                  {wsprintf (Errmsg,TEXT("Error DLL transfer check /StockGrNo"));
                        WinErr ();
                  }
        }
        }
        };
//End of void Init_function_Y1DPackG()
//---------------------------------------------------------

int WinErr ()  // Procedure to display Error messages 
        {
  // Bring up the error dialog box.
         HWND g_herrWnd = NULL;
    int status = MessageBox(g_herrWnd,Errmsg, "Error message", MB_OK );
        if( status == IDOK ) EndDialog (g_herrWnd, 0);
        return 0;
        };
//End of int WinErr ()
//---------------------------------------------------------

void InitVariables()
       {
        int i,j;
                        // === Clear DLL arguments/arrays before call ===
    for(i = 0; i<=CntMax+1; i++)
                {
      StockGrNo[i] = 0;
      StockLayNo[i] = 0;
      StockPatNo[i] = 0;
      StockParts[i] = 0;
      StockWaste[i] = 0;
      StockLength[i] = 0;
      StockCost[i] = 0;
                };

    for(i = 0; i<=BoxMax+1; i++)
                {
      PartGrNo[i] = 0;
      PartStockNo[i] = 0;
      PartLength[i] = 0;
      PartMarkID[i] = 0;
                };

    for(i = 0; i<=CntGrMax+1; i++)
                {
      StockGrLth[i] = 0;
      StockGrCst[i] = 0;
      StockGrQty[i] = 0;
                };

    for(i = 0; i<=BoxGrMax+1; i++)
                {
      PartGrLth[i] = 0;
      PartGrMrkID[i] = 0;
      PartGrQty[i] = 0;
                };

    ErrorNumb = 0;      //=== Clear result variables before call ===
    CritSum = 0;
    EffSum = 0;
    StockSum = 0;
    PartSum = 0;
    LengthSum = 0;
    WasteSum = 0;
    CostSum = 0;
    LastStockNo = 0;
    LastParts = 0;
    LastLength = 0;
    LastWaste = 0;
    LastCost = 0;
    PatternSum = 0;
    LayoutSum = 0;
    LayoutCuts = 0;
    ReservOut1 = 0;   // Reserved
    ReservOut2 = 0;

    ErrorMsgs = 1;  //DLL to display error messages On/Off = 1/0

               //=== Define optimizer settings (DLL input arguments) ===

    CostOptim = 0;    // 0/1 = false/true - stands for boolean
    IgnoreLast = 1;   // 0/1 = false/true - stands for boolean

    PrefShort = 0;    // 0..3 - factor, 0 = No, 3 = max
    ReduceLays = 0;   // 0..3 - factor, 0 = No, 3 = max
    EffortLev = 5;    // 0..5/8/10 - factor

    DiffStocks = 0;   // >=0,  all integers!
    DiffPieces = 0;   // >= 0
    ReusLength = 0;   // >= 0

    GapItIt = 0;      // >= 0, all integers!
    GapItEnd = 0;     // >= 0
    GapGripEnd = 0;   // >= 0

    IterMult = 20;    // 1..20
    OptimTime = 555;  // >= 5
    LayByMark = 0;    // 0/1 = false/true - stands for boolean
    RunMsgMode = 2;   // 0 - Off; 1 - On; 2 - On + Esc abort
    ReservInp2 = 0;   // Reserved

    //*** Define group array input data for DLL-procedure Y1DPackG ***

    //!!! THIS IS THE ONLY PLACE WHERE YOU CAN DEFINE/CORRECT TEST INPUT
    //!!! AND JUST BY GROUPPED INPUT ARRAYS: StockGr**[i] or PartGr**[i]

    StockGrNmb = 1;      //define total number of identical stock groups
    StockGrLth[1] = 350; //define example stock lengths - all integers!
    StockGrCst[1] = 0;   //define example stock costs - all integers!
    StockGrQty[1] = 10;  //define example stock quantities (1-st group)

    PartGrNmb = 3;       //define total number of identical part groups
    PartGrLth[1] = 162;  //define example part lengths - all integers!
    PartGrMrkID[1] = 0;  //define example mark IDs - all integers!
    PartGrQty[1] = 5;    //define example part quantities (1-st group)

    PartGrLth[2] = 94;   // - the same for 2-d part group
    PartGrMrkID[2] = 0;
    PartGrQty[2] = 5;

    PartGrLth[3] = 48;   // - the same for 3-d part group
    PartGrMrkID[3] = 0;
    PartGrQty[3] = 10;
    //!!! NO ANY FURTHER CORRECTIONS for the TEST INPUT ALLOWED ---------

    //*** Derive line array input data for DLL-procedure Y1DPackB ***

    StockNmb = 0;
    for(j = 1; j<=StockGrNmb; j++)
    for(i = 1; i<=StockGrQty[j]; i++)
                {
      StockNmb = StockNmb + 1;  // count total number of stocks to use
      StockLength[StockNmb] = StockGrLth[j];
      StockCost[StockNmb] = StockGrCst[j];
                }  // define test stock lengths and costs (all integers!)
    PartNmb = 0;
    for(j = 1; j<=PartGrNmb; j++)
    for(i = 1; i<=PartGrQty[j]; i++)
                {
      PartNmb = PartNmb + 1;    // count total number of parts to cut
      PartLength[PartNmb] = PartGrLth[j];
      PartMarkID[PartNmb] = PartMarkID[j];
                }  // define test part lengths and mark IDs (all integers!)

      //*** Place array lengths in elem[0]!  (for Y1DPackG input) ***
    StockGrLth[0] = StockGrNmb;
    StockGrCst[0] = StockGrNmb;
    StockGrQty[0] = (short)StockGrNmb;
    PartGrLth[0] = PartGrNmb;
    PartGrMrkID[0] = PartGrNmb;
    PartGrQty[0] = (short)PartGrNmb;
                //place array check codes in elem[n+1]!
    StockGrLth[StockGrNmb + 1] = StockCheck;
    StockGrCst[StockGrNmb + 1] = StockCheck;
    StockGrQty[StockGrNmb + 1] = StockCheck;
    PartGrLth[PartGrNmb + 1] = PartCheck;
    PartGrMrkID[PartGrNmb + 1] = PartCheck;
    PartGrQty[PartGrNmb + 1] = PartCheck;

        //*** Place array lengths in elem[0]! (for Y1DpackB input) ***
    StockLength[0] = StockNmb;
    StockCost[0] = StockNmb;
    PartLength[0] = PartNmb;
    PartMarkID[0] = PartNmb;
                //place array check codes in elem[n+1]!
    StockLength[StockNmb + 1] = StockCheck;
    StockCost[StockNmb + 1] = StockCheck;
    PartLength[PartNmb + 1] = PartCheck;
    PartMarkID[PartNmb + 1] = PartCheck;

    RegCustomer = "Astrokettle Algorithms";
    RegPassword = 5555555;
        }
//End of void InitVariables()
};
//End of class IDSC_Dll
